#include "libgraph.h"
#include "image.h"
#include <alloc.h>
#include <custom.h>
#import <Cocoa/Cocoa.h>

extern NSLock* lock;
extern int width, height;
extern float red, green, blue;
extern BOOL needsRefresh;
extern unsigned char* data;

#define None 0L

void setPixelValue(unsigned char * buffer, int ax, int ay, int n, int w, int h)
{
    
 	if (ax < 0 || ax >= width || ay < 0 || ay >= height) return;   
	int k = (h - ay - 1)*w*4+ax*4;
	buffer[k++] = (n >> 16);
	buffer[k++] = ((n >> 8) & 0xff);
	buffer[k++] = (n & 0xff);
	buffer[k] = 0xff;
}

int getPixelValue(unsigned char * buffer, int ax, int ay, int w, int h)
{
	if (ax < 0 || ax >= width || ay < 0 || ay >= height) return Val_int(16777215); // white pixel
	int k = (h - ay - 1)*w*4+ax*4;
	int r = buffer[k++];
	int g = buffer[k++];
	int b = buffer[k++];
	return (r << 16) + (g << 8) + b;
}

static void caml_gr_free_image(value im)
{
  free(Data_im(im));
}

static struct custom_operations image_ops = {
  "_image",
  caml_gr_free_image,
  custom_compare_default,
  custom_hash_default,
  custom_serialize_default,
  custom_deserialize_default
};

#define Max_image_mem 2000000

value caml_gr_new_image(int w, int h)
{
  value res = caml_alloc_custom(&image_ops, sizeof(struct grimage), w * h, Max_image_mem);
  Width_im(res) = w;
  Height_im(res) = h;
  Data_im(res) = malloc(w*h*4);
  Mask_im(res) = None;
  return res;
}

value caml_gr_create_image(value vw, value vh)
{
  caml_gr_check_open();
  return caml_gr_new_image(Int_val(vw), Int_val(vh));
}

value caml_gr_blit_image(value im, value vx, value vy)
{
	caml_gr_check_open();
	int w = Width_im(im);
	int h = Height_im(im);
	unsigned char* imData = (unsigned char*)Data_im(im);
	int ax = Int_val(vx);
	int ay = Int_val(vy);
	[lock lock];
	
	int i;
	int j;
	for (i = 0;  i < w; i++) {
		for (j = 0;  j < h; j++) {
			if (i + ax < width && j + ay < height) {
				setPixelValue(imData, i, j, getPixelValue(data, i + ax, j + ay, width, height), w, h);				
			}
		}
	}
	
	needsRefresh = YES;
	[lock unlock];
	return Val_unit;	
}

value caml_gr_draw_image(value im, value vx, value vy)
{
	caml_gr_check_open();
	int w = Width_im(im);
	int h	= Height_im(im);
	unsigned char* imData = (unsigned char*)Data_im(im);
	int ax = Int_val(vx);
	int ay = Int_val(vy);
	[lock lock];
	int i;
	int j;
	for (i = 0;  i < w; i++) {
		for (j = 0;  j < h; j++) {
			if (i + ax < width && j + ay < height) {
				setPixelValue(data, i + ax, j + ay, getPixelValue(imData, i, j, w, h), width, height);				
			}
		}
	}
	needsRefresh = YES;
	[lock unlock];
	return Val_unit;
}
